@@ -25,10 +25,14 @@
 
 #include <stdio.h>
 #include <rpcsvc/nis.h>
+#include "nis_print.h"
 
 
 void p_entry_col(entry_col *ecp)
 {
+    if (ecp->ec_value.ec_value_len == 0)
+	return;
+    
     printf("%.*s",
 	   (int) ecp->ec_value.ec_value_len,
 	   ecp->ec_value.ec_value_val);
@@ -49,11 +53,10 @@ void p_entry(entry_obj *eop, int sep)
     putchar('\n');
 }
 
-extern void print_result(nis_result *, int);
 
 int main(int argc, char *argv[])
 {
-    nis_result *nsres;
+    nis_result *res, *nres;
     int separator;
     char *name;
     int verbose = 0;
@@ -79,18 +82,20 @@ int main(int argc, char *argv[])
     else
 	name = argv[1];
     
-    nsres = nis_lookup(name, 0);
+    res = nis_lookup(name, 0);
 
     if (verbose)
     {
 	printf("Result of object lookup:\n");
-	print_result(nsres, 4);
+	nis_print_result(res);
     }
     
-    separator = nsres->objects.objects_val[0].zo_data.objdata_u.ta_data.ta_sep;
-	
-    nsres = nis_first_entry(name);
-    if (nsres == NULL)
+    separator = res->objects.objects_val[0].zo_data.objdata_u.ta_data.ta_sep;
+
+    nis_freeresult(res);
+    
+    res = nis_first_entry(name);
+    if (res == NULL)
     {
 	perror("nis_first_entry");
 	exit(3);
@@ -99,18 +104,19 @@ int main(int argc, char *argv[])
     do
     {
 	if (verbose)
-	    print_result(nsres,4);
+	    nis_print_result(res);
 	else
 	{
 	    int i;
 	    
-	    for (i = 0; i < nsres->objects.objects_len; i++)
-	      p_entry(&nsres->objects.objects_val[0].zo_data.objdata_u.en_data,
+	    for (i = 0; i < res->objects.objects_len; i++)
+	      p_entry(&res->objects.objects_val[i].zo_data.objdata_u.en_data,
 		      separator);
 	}
-    } while ((nsres->status == 0 || nsres->status == 1) &&
-	     (nsres = nis_next_entry(name, &nsres->cookie)));
+    } while ((res->status == 0 || res->status == 1) &&
+	     ((nres = nis_next_entry(name, &res->cookie)),
+	      nis_freeresult(res), res = nres));
 	     
-
     exit(0);
 }
+
