@@ -25,8 +25,8 @@
 
 #include <stdio.h>
 #include <rpcsvc/nis.h>
+#include "nis_print.h"
 
-extern void print_result(nis_result *, int);
 
 int main(argc,argv)
     int argc;
@@ -45,11 +45,13 @@ int main(argc,argv)
     nsres = nis_lookup(argv[1], 0);
     if (nsres == NULL)
     {
-	perror("foo");
+	perror("nis_lookup");
 	exit(3);
     }
 	
-    print_result(nsres, 4);
+    nis_print_result(nsres);
+
+    nis_freeresult(nsres);
     
     exit(0);
 }
