@@ -30,7 +30,9 @@
 #include <stdio.h>
 #include <rpc/rpc.h>
 #include "rpcsvc/nis.h"
+#include "nis_alloc.h"
 #include "nis_conf.h"
+#include "xalloc.h"
 
 
 extern char *strstr();
@@ -76,20 +78,106 @@ int nis_get_default_domain(char **outdomain)
 }
 
 
-nis_name nis_mkfullname(nis_name name, char **domain)
+nis_name nis_parsesearch(nis_name name, int *len, nis_attr **val)
 {
-    static char buf[1024];
+    nis_name rname;
+    char *cp;
+    int size;
+
+
+    if (name == NULL)
+	return NULL;
+    
+    if (len)
+	*len = 0;
+    if (val)
+	*val = NULL;
+
+    size = 0;
+
+    
+    /* Not of "[key=value,key=value,...],foo.." format? */
+    if  (name[0] != '[')
+	return name;
+
+    ++name;
+    cp = strchr(name, ']');
+    if (cp == NULL)
+	return NULL;
+    
+    if (cp[1] == ',')
+	rname = cp+2;
+    else
+	rname = cp+1;
+
+    if (len == NULL || val == NULL)
+	return rname;
+    
+    *cp = '\0';
+
+    cp = strtok(name, ",");
+    while (cp)
+    {
+	name = cp;
+	cp = strchr(name, '=');
+	if (!cp)
+	    return NULL;
+	*cp++ = '\0';
+
+	if ((*len) + 1 >= size)
+	{
+	    size += 10;
+	    if (Xalloc(val, size) == NULL)
+		return NULL;
+	}
+
+	(*val)[*len].zattr_ndx = xstrdup(name);
+	if (((*val)[*len].zattr_ndx) == NULL)
+	    return NULL;
+	    
+	(*val)[*len].zattr_val.zattr_val_len = strlen(cp)+1;
+	
+	(*val)[*len].zattr_val.zattr_val_val = xstrdup(cp);
+	if ((*val)[*len].zattr_val.zattr_val_val == NULL)
+	    return NULL;
+	
+	(*len)++;
+	
+	cp = strtok(NULL, ",");
+    }
+
+    return rname;
+}
+
+nis_name nis_parsename(nis_name name,
+		       char **domain,
+		       int *srchlen,
+		       nis_attr **srchval)
+{
+    static char *pbuf = NULL;
     int len;
     char *ddom;
     
+
+    if (name == NULL)
+	return NULL;
     
     len = strlen(name);
-    strcpy(buf, name);
+    if (Xalloc(&pbuf, len+1) == NULL)
+	return NULL;
+    
+    strcpy(pbuf, name);
     
     if (len == 0)
-	return buf;
+	return pbuf;
 
-    if (name[len-1] == '.')
+    name = nis_parsesearch(pbuf, srchlen, srchval);
+    
+    if (name == NULL || (len = strlen(name)) == 0)
+	return NULL;
+
+    strcpy(pbuf, name);
+    if (pbuf[len-1] == '.')
     {
 	/* Fully specified domainname */
 
@@ -100,11 +188,11 @@ nis_name nis_mkfullname(nis_name name, char **domain)
 
 	    
 	    i = sizeof(".org_dir.");
-	    cp = strstr(name, ".org_dir.");
+	    cp = strstr(pbuf, ".org_dir.");
 	    if (cp == NULL)
 	    {
 		i = sizeof(".group_dir.");
-		cp = strstr(name, ".group_dir.");
+		cp = strstr(pbuf, ".group_dir.");
 	    }
 
 	    if (cp == NULL)
@@ -112,21 +200,24 @@ nis_name nis_mkfullname(nis_name name, char **domain)
 	    else
 		*domain = cp + i - 1;
 	}
-	return buf;
+	return pbuf;
     }
     
     if (nis_get_default_domain(&ddom) == 0)
     {
-	strcat(buf, ".");
-	strcat(buf, ddom);
+	if (Xalloc(&pbuf, len+strlen(ddom)+2) == NULL)
+	    return NULL;
+	
+	strcat(pbuf, ".");
+	strcat(pbuf, ddom);
 	
 	if (domain)
 	    *domain = ddom;
 	
-         return buf;
+         return pbuf;
     }
     else 
-	return buf; /* XXX or should we return NULL and fail? */
+	return NULL;
 }
 
 
@@ -163,8 +254,7 @@ static nis_result *
 	fprintf(stderr, "NIS: server \"%s\" OK.\n",
 		address);
 
-    return res;
-		
+    return nis_dupresult(res);
 }
 
 
@@ -177,7 +267,7 @@ nis_result *nis_lookup(nis_name name, u_long flags)
     if (nis_setup())
 	return NULL;
 
-    nsreq.ns_name = nis_mkfullname(name, &domain);
+    nsreq.ns_name = nis_parsename(name, &domain, NULL, NULL);
     nsreq.ns_object.ns_object_len = 0;
     nsreq.ns_object.ns_object_val = NULL;
 	
@@ -188,18 +278,20 @@ nis_result *nis_lookup(nis_name name, u_long flags)
 nis_result *nis_first_entry(nis_name name)
 {
     struct ib_request ibreq;
+    nis_result *res;
     char *domain;
 
     
     if (nis_setup() == -1)
 	return NULL;
-
-    ibreq.ibr_name = nis_mkfullname(name, &domain);
+    
+    ibreq.ibr_name = nis_parsename(name,
+				   &domain,
+				   &ibreq.ibr_srch.ibr_srch_len,
+				   &ibreq.ibr_srch.ibr_srch_val);
+				   
     ibreq.ibr_flags = 0;
 
-    ibreq.ibr_srch.ibr_srch_len = 0;
-    ibreq.ibr_srch.ibr_srch_val = NULL;
-    
     ibreq.ibr_obj.ibr_obj_len = 0;
     ibreq.ibr_obj.ibr_obj_val = NULL;
     
@@ -211,12 +303,19 @@ nis_result *nis_first_entry(nis_name name)
     ibreq.ibr_cookie.n_len = 0;
     ibreq.ibr_cookie.n_bytes = NULL;
     
-    return do_niscall(domain, &nis_ibfirst_3, &ibreq);
+    res = do_niscall(domain, &nis_ibfirst_3, &ibreq);
+
+    if (ibreq.ibr_srch.ibr_srch_val)
+	nis_freeattrs(ibreq.ibr_srch.ibr_srch_val,
+		      ibreq.ibr_srch.ibr_srch_len);
+
+    return res;
 }
 
 
 nis_result *nis_next_entry(nis_name name, netobj *cookie)
 {
+    nis_result *res;
     struct ib_request ibreq;
     char *domain;
     
@@ -224,11 +323,11 @@ nis_result *nis_next_entry(nis_name name, netobj *cookie)
     if (nis_setup() == -1)
 	return NULL;
 
-    ibreq.ibr_name = nis_mkfullname(name, &domain);
+    ibreq.ibr_name = nis_parsename(name,
+				   &domain,
+				   &ibreq.ibr_srch.ibr_srch_len,
+				   &ibreq.ibr_srch.ibr_srch_val);
     ibreq.ibr_flags = 0;
-
-    ibreq.ibr_srch.ibr_srch_len = 0;
-    ibreq.ibr_srch.ibr_srch_val = NULL;
     
     ibreq.ibr_obj.ibr_obj_len = 0;
     ibreq.ibr_obj.ibr_obj_val = NULL;
@@ -240,7 +339,13 @@ nis_result *nis_next_entry(nis_name name, netobj *cookie)
     
     ibreq.ibr_cookie = *cookie;
 
-    return do_niscall(domain, &nis_ibnext_3, &ibreq);
+    res = do_niscall(domain, &nis_ibnext_3, &ibreq);
+
+    if (ibreq.ibr_srch.ibr_srch_val)
+	nis_freeattrs(ibreq.ibr_srch.ibr_srch_val,
+		      ibreq.ibr_srch.ibr_srch_len);
+
+    return res;
 }
 
 #endif /* ENABLE_NISEMU */
