@@ -36,21 +36,22 @@
 #include "yp_misc.h"
 #include "yp_networks.h"
 #include "rpcsvc/ypclnt.h"
+#include "xalloc.h"
+
+
+static char *pbuf = NULL;
 
 
 static struct netent *netent_parse(char *str, int len)
 {
     static struct netent netd;
     extern char *strtok();
-    static char buf[2048];
-    
 
-    /* Can't handle this -> let's just forget it... */
-    if (len > 2040)
+    
+    if (Xalloc(&pbuf, len) == NULL)
 	return NULL;
 
-    /* XXX to be done XXX */
-    strncpy(buf, str, len);
+    strncpy(pbuf, str, len);
     
     return NULL;
 }
@@ -61,7 +62,7 @@ static char *savekey = NULL;
 static int savekeylen = 0;
 
 
-void yp_setnetent(char *opts)
+void yp_setnetent(int stayopen, void *opts)
 {
     rewind_flag = 1;
     if (savekey)
@@ -69,7 +70,7 @@ void yp_setnetent(char *opts)
 }
 
 
-void yp_endnetent(char *opts)
+void yp_endnetent(void *opts)
 {
     rewind_flag = 1;
     if (savekey)
@@ -77,7 +78,7 @@ void yp_endnetent(char *opts)
 }
 
 
-struct netent *yp_getnetent(char *opts)
+struct netent *yp_getnetent(void *opts)
 {
     struct netent *net;
     char *map;
@@ -128,14 +129,14 @@ struct netent *yp_getnetent(char *opts)
 }
 
 
-struct netent *yp_getnetbyaddr(long net, int type, char *opts)
+struct netent *yp_getnetbyaddr(long net, int type, void *opts)
 {
     struct netent *netd;
     char *map;
     char *domain;
     char *result;
     int len;
-    char buf[256];
+    char buf[16];
 
 
     map = _ypopts_getmd(opts, "networks", ".byaddr", &domain);
@@ -159,7 +160,7 @@ struct netent *yp_getnetbyaddr(long net, int type, char *opts)
 }
 
 
-struct netent *yp_getnetbyname(char *name, char *opts)
+struct netent *yp_getnetbyname(char *name, void *opts)
 {
     struct netent *net;
     char *map;
